@@ -28,8 +28,10 @@
 #include "minmax.h"		/* Gnulib */
 #include "xalloc.h"		/* Gnulib */
 #include "quotearg.h"		/* Gnulib */
+#include "dirname.h"		/* Gnulib */
 #include "strbuf.h"
 #include "intutil.h"
+#include "strutil.h"
 #include "gmediaserver.h"
 #include "search-parser.h"
 #include "search-lexer.h"
@@ -115,7 +117,9 @@ begin_result(StrBuf *result)
 static char *
 get_entry_property(Entry *entry, char *property)
 {
-    EntryDetail *tag_detail;
+    EntryDetail *detail;
+
+    detail = get_entry_detail(entry, DETAIL_TAG);
 
     /* XXX: should we allow namespace-less property names,
      * such as "title" rather than "dc:title"?
@@ -123,26 +127,27 @@ get_entry_property(Entry *entry, char *property)
      * such as @refID, @id, @parentID or @protocolInfo?
      */
 
-    tag_detail = get_entry_detail(entry, DETAIL_TAG);
     if (strcmp(property, "dc:title") == 0) {
-	if (tag_detail != NULL && tag_detail->data.tag.title != NULL)
-	    return tag_detail->data.tag.title;
-	return entry->filename;
+	if (detail != NULL && detail->data.tag.title != NULL && strcmp(detail->data.tag.title, "") != 0)
+	    return convert_string(detail->data.tag.title);
+        return convert_string(entry->name);
     }
 
     if (strcmp(property, "upnp:class") == 0) {
 	if (has_entry_detail(entry, DETAIL_CHILDREN))
-	    return "object.container.storageFolder";
-	return "object.item.audioItem.musicTrack";
+	    return convert_string("object.container.storageFolder");
+        /*if (has_entry_detail(entry, DETAIL_URL))
+            return convert_string("object.item.audioItem.audioBroadcast");*/
+	return convert_string("object.item.audioItem.musicTrack");
     }
-    
-    if (tag_detail != NULL) {
+
+    if (detail != NULL) {
 	if (strcmp(property, "upnp:artist") == 0)
-	    return tag_detail->data.tag.artist;
+	    return convert_string(detail->data.tag.artist);
 	if (strcmp(property, "upnp:album") == 0)
-	    return tag_detail->data.tag.album;
+	    return convert_string(detail->data.tag.album);
 	if (strcmp(property, "upnp:genre") == 0)
-	    return tag_detail->data.tag.genre;
+	    return convert_string(detail->data.tag.genre);
     }
 
     return NULL;
@@ -154,7 +159,7 @@ filter_includes(const char *filter, const char *field)
     if (strcmp(filter, "*") == 0)
         return true;
 
-    if (string_in_csv(filter, field))
+    if (string_in_csv(filter, ',', field))
 	return true;
 
     return false;
@@ -166,8 +171,8 @@ append_entry(StrBuf *result, Entry *entry, const char *filter)
     EntryDetail *detail;
     char *value;
 
-    detail = get_entry_detail(entry, DETAIL_CHILDREN);
-    if (detail != NULL) {
+    if (has_entry_detail(entry, DETAIL_CHILDREN)) {
+        detail = get_entry_detail(entry, DETAIL_CHILDREN);
 	strbuf_appendf(result,
 		"<container id=\"%" PRIi32 "\" parentID=\"%" PRIi32 "\" restricted=\"true\" childCount=\"%" PRIi32 "\">",
 		entry->id,
@@ -179,66 +184,68 @@ append_entry(StrBuf *result, Entry *entry, const char *filter)
 		entry->id,
 		entry->parent);
         if (filter_includes(filter, "res") || filter_includes(filter, "@protocolInfo") || filter_includes(filter, "@size")) {
-            strbuf_append(result, "<res");
-            if (filter_includes(filter, "res") || filter_includes(filter, "@protocolInfo"))
-                strbuf_appendf(result, " protocolInfo=\"http-get:*:%s:*\"", entry->mime_type);
-	    /*if (entry->size != 0)*/
-	    if (filter_includes(filter, "res") || filter_includes(filter, "@size"))
-	        strbuf_appendf(result, " size=\"%" PRIu32 "\"", entry->size);
-            strbuf_appendf(result, ">http://%s:%d/files/%d</res>",
-		    UpnpGetServerIpAddress(),
-		    UpnpGetServerPort(),
-		    entry->id);
+            detail = get_entry_detail(entry, DETAIL_FILE);
+            if (detail != NULL) {
+                strbuf_append(result, "<res");
+                if (filter_includes(filter, "res") || filter_includes(filter, "@protocolInfo"))
+                    strbuf_appendf(result, " protocolInfo=\"http-get:*:%s:*\"", detail->data.file.mime_type);
+                /*if (detail->data.file.size != 0)*/
+                if (filter_includes(filter, "res") || filter_includes(filter, "@size"))
+                    strbuf_appendf(result, " size=\"%" PRIu32 "\"", detail->data.file.size);
+                strbuf_appendf(result, ">http://%s:%d/files/%d</res>",
+                        UpnpGetServerIpAddress(),
+                        UpnpGetServerPort(),
+                        entry->id);
+            } else {
+                detail = get_entry_detail(entry, DETAIL_URL);
+                strbuf_append(result, "<res");
+                if (filter_includes(filter, "res") || filter_includes(filter, "@protocolInfo"))
+                    strbuf_appendf(result, " protocolInfo=\"http-get:*:audio/mpeg:*\""); /* FIXME: application/octet-stream? */
+                strbuf_appendf(result, ">http://%s:%d/files/%d</res>",
+                        UpnpGetServerIpAddress(),
+                        UpnpGetServerPort(),
+                        entry->id);
+            }
         }
     }
 
     /* upnp:class is required and cannot be filtered out. */
-    strbuf_appendf(result, "<upnp:class>%s</upnp:class>", get_entry_property(entry, "upnp:class"));
+    value = get_entry_property(entry, "upnp:class");
+    strbuf_appendf(result, "<upnp:class>%s</upnp:class>", value);
+    free(value);
 
     /* dc:title is required and cannot be filtered out. */
     value = get_entry_property(entry, "dc:title");
     if (value != NULL)
         append_escaped_xml(result, "dc:title", value);
+    free(value);
     /* XXX: dc:creator instead or in combination with upnp:artist? */
     if (filter_includes(filter, "upnp:artist")) {
         value = get_entry_property(entry, "upnp:artist");
         if (value != NULL)
 	    append_escaped_xml(result, "upnp:artist", value);
+        free(value);
     }
     if (filter_includes(filter, "upnp:album")) {
         value = get_entry_property(entry, "upnp:album");
         if (value != NULL)
 	    append_escaped_xml(result, "upnp:album", value);
+        free(value);
     }
     if (filter_includes(filter, "upnp:genre")) {
         value = get_entry_property(entry, "upnp:genre");
         if (value != NULL)
 	    append_escaped_xml(result, "upnp:genre", value);
+        free(value);
     }
 
-    if (detail != NULL) {
+    if (has_entry_detail(entry, DETAIL_CHILDREN)) {
 	strbuf_append(result, "</container>");
     } else {
 	strbuf_append(result, "</item>");
     }
 }
 
-/* XXX: move this to util.c or strutil.c */
-char *
-xstrdupn(const char *str, size_t n)
-{
-    char *out;
-    size_t len;
-
-    len = strlen(str);
-    out = xmalloc(n*len+1);
-    out[n*len] = '\0';
-    for (; n > 0; n--)
-	memcpy(out+(n-1)*len, str, len);
-
-    return out;
-}
-
 static char *
 operator_name(int type)
 {
@@ -320,6 +327,7 @@ static bool
 match_search_expr(SearchExpr *expr, Entry *entry)
 {
     char *value;
+    bool result;
     
     switch (expr->type) {
     case T_AND:
@@ -329,32 +337,50 @@ match_search_expr(SearchExpr *expr, Entry *entry)
 	return match_search_expr(expr->u.logical.expr1, entry)
 		|| match_search_expr(expr->u.logical.expr2, entry);
     case T_EXISTS:
-	return (get_entry_property(entry, expr->u.exists.property) != NULL)
-		== expr->u.exists.must_exist;
+        value = get_entry_property(entry, expr->u.exists.property);
+        result = (value != NULL) == (expr->u.exists.must_exist);
+        free(value);
+        return result;
     case T_EQ:
 	value = get_entry_property(entry, expr->u.binary.property);
-	return strcmp(value, expr->u.binary.value) == 0;
+	result = strcmp(value, expr->u.binary.value) == 0;
+	free(value);
+	return result;
     case T_NE:
 	value = get_entry_property(entry, expr->u.binary.property);
-	return strcmp(value, expr->u.binary.value) != 0;
+	result = strcmp(value, expr->u.binary.value) != 0;
+	free(value);
+	return result;
     case T_LT:
 	value = get_entry_property(entry, expr->u.binary.property);
-	return strcmp(value, expr->u.binary.value) > 0;
+	result = strcmp(value, expr->u.binary.value) > 0;
+	free(value);
+	return result;
     case T_LE:
 	value = get_entry_property(entry, expr->u.binary.property);
-	return strcmp(value, expr->u.binary.value) >= 0;
+	result = strcmp(value, expr->u.binary.value) >= 0;
+	free(value);
+	return result;
     case T_GT:
 	value = get_entry_property(entry, expr->u.binary.property);
-	return strcmp(value, expr->u.binary.value) < 0;
+	result = strcmp(value, expr->u.binary.value) < 0;
+	free(value);
+	return result;
     case T_GE:
 	value = get_entry_property(entry, expr->u.binary.property);
-	return strcmp(value, expr->u.binary.value) <= 0;
+	result = strcmp(value, expr->u.binary.value) <= 0;
+	free(value);
+	return result;
     case T_CONTAINS:
 	value = get_entry_property(entry, expr->u.binary.property);
-	return strstr(value, expr->u.binary.value) != NULL;
+	result = strstr(value, expr->u.binary.value) != NULL;
+	free(value);
+	return result;
     case T_DOES_NOT_CONTAIN:
 	value = get_entry_property(entry, expr->u.binary.property);
-	return strstr(value, expr->u.binary.value) == NULL;
+	result = strstr(value, expr->u.binary.value) == NULL;
+	free(value);
+	return result;
     case T_DERIVED_FROM:
 	/* XXX: clean up these comparisons, can be made simpler! */
 	if (strcmp(expr->u.binary.value, "object") == 0)
