@@ -26,22 +26,31 @@
 #include <stdio.h>		/* C89; FILE */
 #include <string.h>		/* C89 */
 #include <stdbool.h>		/* Gnulib, C99; bool */
+#include <sys/types.h>		/* POSIX: time_t */
 #include <upnp.h>		/* libupnp */
+#include "hmap.h"
 #include "strbuf.h"
 
 typedef struct _ServiceAction ServiceAction;
 typedef struct _Service Service;
 typedef struct _ActionEvent ActionEvent;
+typedef struct _EntryChildren EntryChildren;
+typedef struct _EntryTag EntryTag;
 typedef struct _EntryDetail EntryDetail;
+typedef struct _EntryFile EntryFile;
+typedef struct _EntryURL EntryURL;
 typedef struct _Entry Entry;
 typedef struct _SearchExpr SearchExpr;
 typedef struct _SearchCriteria SearchCriteria;
 typedef struct _SearchCriteriaParseData SearchCriteriaParseData;
+typedef struct _HTTPResult HTTPResult;
+typedef struct _URL URL;
 
 typedef enum {
-    DETAIL_CHILDREN,
-    DETAIL_LOCAL_PATH,
-    DETAIL_TAG,
+    DETAIL_CHILDREN,	/* mutually exclusive with DETAIL_FILE, URL */
+    DETAIL_TAG,		/* currently only available for DETAIL_FILE */
+    DETAIL_FILE,	/* mutually exclusive with DETAIL_CHILDREN, URL */
+    DETAIL_URL,		/* mutually exclusive with DETAIL_CHILDREN, FILE */
 } EntryDetailType;
 
 struct _ServiceAction {
@@ -61,33 +70,46 @@ struct _ActionEvent {
     Service *service;
 };
 
+struct _EntryChildren {
+    uint32_t count;
+    int32_t *list;
+};
+
+struct _EntryTag {
+    char *title; /* if set and non-empty, title replaces entry->name in display */
+    char *artist;
+    char *album;
+    char *genre;
+};
+
+struct _EntryURL {
+    char *url;
+};
+
+struct _EntryFile {
+    const char *mime_type;
+    size_t size;
+    char *filename;
+    time_t mtime;
+};
+
 struct _EntryDetail {
     EntryDetail *next;
     /*EntryDetail *previous;*/
     EntryDetailType type;
     union {
-	char *local_path;
-	struct {
-	    char *title;
-	    char *artist;
-	    char *album;
-	    char *genre;
-	} tag;
-	struct {
-	    uint32_t count;
-	    int32_t *list;
-	} children;
+	EntryTag tag;
+	EntryChildren children;
+	EntryFile file;
+	EntryURL url;
     } data;
 };
 
 struct _Entry {
     int32_t id;
     int32_t parent;
-    char *filename;
-    const char *mime_type;
-    uint32_t size;
+    char *name; /* to display in control point */
     EntryDetail *details;
-    /*uint32_t detail_flags; 		unused for now */
 };
 
 struct _SearchExpr {
@@ -122,19 +144,37 @@ struct _SearchCriteriaParseData {
     const char *error;
 };
 
+struct _HTTPResult {
+    bool icy_server;
+    int result;
+    HMap *headers;
+    int fd;
+    char *buf;
+    size_t bufsize;
+    size_t bufhead;
+    size_t buftail;
+};
+
+struct _URL {
+    /*char *protocol;*/
+    char *hostname;
+    uint16_t port;
+    char *path;
+};
+
 /* logging.c */
 #define DEFAULT_TIMESTAMP_FORMAT "%Y-%m-%d %H:%M"
 #define MAX_VERBOSITY 4
 #define errstr strerror(errno)
 extern uint32_t verbosity;
-void say(int level, const char *format, ...);
-void die(const char *format, ...);
-void warn(const char *format, ...);
+void say(int level, const char *format, ...) __attribute__ ((format (printf, 2, 3)));
+void die(const char *format, ...) __attribute__ ((format (printf, 1, 2)));
+void warn(const char *format, ...) __attribute__ ((format (printf, 1, 2)));
 void init_logging(const char *logfilename, const char *timestamp_format);
 void finish_logging(bool success);
 
 /* main.c */
-bool string_in_csv(const char *csv, const char *value);
+char *convert_string(const char *str);
 
 /* interface.h */
 char *get_ip_by_spec(char *spec);
@@ -147,7 +187,6 @@ extern bool id3_enabled;
 bool init_metadata(char **pathv, int pathc);
 void finish_metadata(void);
 Entry *get_entry_by_id(uint32_t id);
-char *get_entry_path(Entry *entry);
 EntryDetail *get_entry_detail(Entry *entry, EntryDetailType type);
 bool has_entry_detail(Entry *entry, EntryDetailType type);
 
@@ -176,7 +215,7 @@ extern char *friendly_name;
 extern int ssdp_expire_time;
 void init_upnp(const char *listenip, uint16_t listenport);
 void finish_upnp(void);
-void upnp_set_error(ActionEvent *event, int error_code, const char *format, ...);
+void upnp_set_error(ActionEvent *event, int error_code, const char *format, ...) __attribute__ ((format (printf, 3, 4)));
 char *upnp_get_string(ActionEvent *event, const char *item);
 uint32_t upnp_get_ui4(ActionEvent *event, const char *key);
 bool upnp_add_response(ActionEvent *event, char *key, const char *value);
@@ -185,6 +224,13 @@ int32_t upnp_get_i4(ActionEvent *event, const char *key);
 /* webserver.c */
 extern struct UpnpVirtualDirCallbacks virtual_dir_callbacks;
 
+/* webclient.c */
+HTTPResult *http_query(const char *method, const char *url, bool keep_open);
+void http_result_free(HTTPResult *result);
+ssize_t http_skip(HTTPResult *result, size_t count);
+ssize_t http_read(HTTPResult *result, void *buf, size_t count);
+bool parse_http_date(const char *s, time_t *timeptr);
+
 /* contentdir.c */
 extern ServiceAction contentdir_service_actions[];
 void free_search_criteria(SearchCriteria *criteria);
@@ -198,5 +244,7 @@ void sgmlescape(const char *str, char *target, uint32_t *length);
 char *xsgmlescape(const char *str);
 void urlencode(const char *str, char *target, uint32_t *length);
 char *xurlencode(const char *str);
+URL *parse_simple_url(const char *url);
+void free_url(URL *url);
 
 #endif
