@@ -22,12 +22,12 @@
 #include <unistd.h>		/* POSIX */
 #include <stdio.h>		/* C89 */
 #include <string.h>		/* C89 */
-#include <net/if.h>		/* ? */
+#include <sys/socket.h>		/* POSIX */
 #include <sys/ioctl.h>		/* ? */
+#include <sys/param.h>		/* ? */
+#include <net/if.h>		/* ? */
 #include <netinet/in.h>		/* ? */
 #include <arpa/inet.h>		/* ? */
-#include <sys/param.h>		/* ? */
-#include <sys/socket.h>		/* ? */
 #include "gmediaserver.h"
 
 char *
