@@ -31,6 +31,11 @@
 #include <stdarg.h>		/* C89 */
 #include <stdbool.h>		/* Gnulib, C99 */
 #include <signal.h>		/* ? */
+#include <iconv.h>		/* Gnulib, POSIX */
+#include "iconvme.h"		/* Gnulib */
+#ifdef HAVE_NL_LANGINFO
+#include <langinfo.h>
+#endif
 #include "gettext.h"		/* Gnulib/gettext */
 #define _(s) gettext(s)
 #define N_(s) gettext_noop(s)
@@ -38,12 +43,16 @@
 #include "version-etc.h"	/* Gnulib */
 #include "progname.h"		/* Gnulib */
 #include "sig2str.h"		/* Gnulib */
+#include "quote.h"		/* Gnulib */
 #include "tempfailure.h"
 #include "intutil.h"
+#include "strutil.h"
 #include "gmediaserver.h"
 
 enum {
     OPT_DISABLE_ID3,
+    OPT_IN_CHARSET,
+    OPT_DEVICE_CHARSET,
     OPT_FRIENDLY_NAME,
     OPT_PIDFILE,
     OPT_PORT,
@@ -56,12 +65,13 @@ enum {
     OPT_VERSION,
 };
 
-#define SHORT_OPTIONS "bv::i:o:p:"
-
+static const char *short_options = "bv::i:o:p:";
 static struct option long_options[] = {
 #ifdef HAVE_ID3LIB
     { "disable-id3", no_argument, NULL, OPT_DISABLE_ID3 },
 #endif
+    { "in-charset", required_argument, NULL, OPT_IN_CHARSET },
+    { "device-charset", required_argument, NULL, OPT_DEVICE_CHARSET },
     { "friendly-name", required_argument, NULL, OPT_FRIENDLY_NAME },
     { "pid-file", required_argument, NULL, OPT_PIDFILE },
     { "profile", required_argument, NULL, OPT_PROFILE, },
@@ -79,23 +89,15 @@ static struct option long_options[] = {
     { NULL, 0, NULL, 0 }
 };
 
+iconv_t charset_convert = (iconv_t) -1;
 const char version_etc_copyright[] = "Copyright (C) 2005 Oskar Liljeblad.";
 
-/* XXX: move this to strutil.c or something */
-bool
-string_in_csv(const char *csv, const char *value)
+char *
+convert_string(const char *str)
 {
-    const char *p0;
-    const char *p1;
-
-    for (p0 = csv; (p1 = strchr(p0, ',')) != NULL; p0 = p1+1) {
-        if (strncmp(p0, value, p1-p0) == 0)
-            return true;
-    }
-    if (strcmp(p0, value) == 0)
-        return true;
-
-    return false;
+    if (charset_convert == (iconv_t) -1)
+        return xstrdup(str);
+    return iconv_alloc(charset_convert, str);
 }
 
 static void
@@ -115,12 +117,12 @@ write_pid_file(const char *pidfilename, pid_t pid, bool background)
          * write, which is pretty serious though.
          */
 	if (!background)
-	    say(2, _("Writing pid to pid file `%s'\n"), pidfilename);
+	    say(2, _("Writing pid to pid file %s\n"), quote(pidfilename));
 	if ((pidfh = fopen(pidfilename, "w")) == NULL)
-	    die(_("Cannot open pid file `%s' for writing - %s\n"), pidfilename, errstr);
+	    die(_("cannot open pid file %s for writing - %s\n"), quote(pidfilename), errstr);
 	fprintf(pidfh, "%d", pid);
 	if (fclose(pidfh) != 0)
-	    die(_("Cannot close pid file `%s' - %s\n"), pidfilename, errstr);
+	    die(_("cannot close pid file %s - %s\n"), quote(pidfilename), errstr);
     }
 }
 
@@ -137,20 +139,31 @@ main(int argc, char **argv)
     char *logfilename = NULL;
     char *timestamp_format = NULL;
     uint32_t expire_time;
+    char *in_charset = NULL;
+    char *device_charset = NULL;
 
     set_program_name(argv[0]);
 
+    if (setlocale(LC_ALL, "") == NULL)
+        warn(_("cannot set locale: %s\n"), errstr);
+#ifdef ENABLE_NLS
+    if (bindtextdomain(PACKAGE, LOCALEDIR) == NULL)
+        warn(_("cannot bind message domain: %s\n"), errstr);
+    if (textdomain(PACKAGE) == NULL)
+        warn(_("cannot set message domain: %s\n"), errstr);
+#endif
+
     for (;;) {
     	int c;
 
-        c = getopt_long(argc, argv, SHORT_OPTIONS, long_options, NULL);
+        c = getopt_long(argc, argv, short_options, long_options, NULL);
         if (c == -1)
             break;
 
         switch (c) {
         case 'i':
 	    if ((listenip = get_ip_by_spec(optarg)) == NULL)
-	    	die(_("invalid address or interface name `%s'\n"), optarg);
+	    	die(_("invalid address or interface name %s\n"), quote(optarg));
             break;
         case 'o':
             logfilename = optarg;
@@ -160,6 +173,12 @@ main(int argc, char **argv)
 	    id3_enabled = false;
 	    break;
 #endif
+        case OPT_IN_CHARSET:
+            in_charset = optarg;
+            break;
+        case OPT_DEVICE_CHARSET:
+            device_charset = optarg;
+            break;
 	case OPT_FRIENDLY_NAME:
 	    if (optarg[0] == '\0')
 		die(_("friendly name cannot be empty\n"));
@@ -167,18 +186,18 @@ main(int argc, char **argv)
 	    break;
 	case OPT_FILE_TYPES:
 	    file_types = optarg;
-	    /* FIXME: check valid */
+	    /* XXX: check that list is valid */
 	    break;
 	case OPT_PIDFILE:
 	    pidfilename = optarg;
 	    break;
 	case 'p':
 	    if (!parse_uint16(optarg, &listenport))
-		die(_("invalid port number `%s'\n"), optarg);
+		die(_("invalid port number %s\n"), quote(optarg));
 	    break;
         case OPT_EXPIRE_TIME:
             if (!parse_uint32(optarg, &expire_time) || expire_time > INT_MAX)
-		die(_("invalid expire time `%s'\n"), optarg);
+		die(_("invalid expire time %s\n"), quote(optarg));
             ssdp_expire_time = expire_time;
             /* XXX: parse better, allow hours etc? */
             break;
@@ -192,14 +211,14 @@ main(int argc, char **argv)
 	    break;
 	case OPT_PROFILE:
 	    if (strcmp(optarg, "generic") == 0) {
-		/* no operation */
+		/* no operation - no custom settings*/
 	    } else if (strcmp(optarg, "mp101") == 0) {
-	        file_types = "mp3,wma";
+	        file_types = "mp3,wma,m3u,pls";
 	    } else if (strcmp(optarg, "dms1") == 0) {
-	        file_types = "mp3,wma,m4a,wav";
-		expire_time = 1801;
+	        file_types = "mp3,wma,m3u,pls";
+		expire_time = 600;
 	    } else {
-		die(_("invalid profile name `%s'\n"), optarg);
+		die(_("invalid profile name %s\n"), quote(optarg));
 	    }
 	    break;
         case 'b':
@@ -210,12 +229,12 @@ main(int argc, char **argv)
 		if (optarg[0] == 'v') {
 		    for (verbosity = 0; optarg[verbosity] != '\0'; verbosity++) {
 			if (optarg[verbosity] != 'v')
-			    die(_("invalid verbosity `%s'\n"), optarg);
+			    die(_("invalid verbosity %s\n"), quote(optarg));
 		    }
 		    verbosity++;
 		} else {
 		    if (!parse_uint32(optarg, &verbosity))
-			die(_("invalid verbosity `%s'\n"), optarg);
+			die(_("invalid verbosity %s\n"), quote(optarg));
 		}
 		if (verbosity > MAX_VERBOSITY)
 		    die(_("maximum verbosity is %d\n"), MAX_VERBOSITY);
@@ -246,7 +265,7 @@ main(int argc, char **argv)
 		     "See strftime(3) for description of the timestamp format.\n"));
 	    printf(_("\nA profile specified with --profile enables settings for a certain device.\n"
 		     "The following values are valid for --profile:\n\n"));
-	    printf(_("  generic  no special settings\n"
+	    printf(_("  generic  no custom settings\n"
 	             "  mp101    Netgear MP101\n"
 		     "  dms1     Omnifi DMS1\n"));
             printf(_("\nReport bugs to <%s>.\n"), PACKAGE_BUGREPORT);
@@ -264,15 +283,13 @@ main(int argc, char **argv)
         fprintf(stderr, _("Try `%s --help' for more information.\n"), argv[0]);
         exit(1);
     }
-    if (!timestamp_specified && logfilename != NULL)
-        timestamp_format = DEFAULT_TIMESTAMP_FORMAT;
 
     if (background) {
         pid_t pid;
 
         /*say(2, "Moving process into background...\n");*/
     	if ((pid = fork()) < 0)
-    	    die(_("Cannot fork new process - %s\n"), errstr);
+    	    die(_("cannot create new process - %s\n"), errstr);
         if (pid != 0) {
             /* Write pid before we return control to the shell. */
             write_pid_file(pidfilename, pid, true);
@@ -287,20 +304,38 @@ main(int argc, char **argv)
         close(STDERR_FILENO);	/* ignore errors */
 
         if (setsid() < 0)
-	    die(_("Cannot create process session - %s\n"), errstr);
+	    die(_("cannot create process session - %s\n"), errstr);
 
 	/* XXX: The following is necessary to work around a bug in
 	 *      libupnp 1.2.1 - fd 0 must not be closed!
 	 */
 	if (open("/dev/null", O_RDONLY) < 0)
-	    die(_("Cannot open /dev/null - %s\n"), errstr);
+	    die(_("cannot open /dev/null - %s\n"), errstr);
     }
 
+
     if (!timestamp_specified && logfilename != NULL)
     	timestamp_format = DEFAULT_TIMESTAMP_FORMAT;
 
     init_logging(logfilename, timestamp_format);
 
+    if (device_charset != NULL) {
+        if (in_charset == NULL) {
+#ifdef HAVE_NL_LANGINFO
+            in_charset = nl_langinfo(CODESET);
+            if (in_charset == NULL)
+                die(_("cannot get current character set - please specify with --in-charset\n"));
+#else
+            die(_("--in-charset must be specified with --device-charset\n"));
+#endif
+        }
+        say(4, "Using input character set %s\n", quote(in_charset));
+        say(4, "Using device character set %s\n", quote(device_charset));
+        charset_convert = iconv_open(device_charset, in_charset);
+        if (charset_convert == (iconv_t) -1)
+            die(_("cannot create character set convertor\nTry using another value for --in-charset or --device-charset\n"));
+    }
+
     /* We could write pid before initiating logging too.
      */
     if (!background)
@@ -308,28 +343,28 @@ main(int argc, char **argv)
 
     signalaction.sa_handler = dummy_signal_handler;
     if (sigemptyset(&signalaction.sa_mask) < 0)
-    	die(_("Cannot empty signal set - %s\n"), errstr);
+    	die(_("cannot empty signal set - %s\n"), errstr);
     signalaction.sa_flags = SA_RESTART;
 #ifdef HAVE_STRUCT_SIGACTION_SA_RESTORER
     signalaction.sa_restorer = NULL;
 #endif
     if (sigaction(SIGINT, &signalaction, NULL) < 0)
-    	die(_("Cannot register signal handler - %s\n"), errstr);
+    	die(_("cannot register signal handler - %s\n"), errstr);
     if (sigaction(SIGTERM, &signalaction, NULL) < 0)
-    	die(_("Cannot register signal handler - %s\n"), errstr);
+    	die(_("cannot register signal handler - %s\n"), errstr);
     if (sigaction(SIGUSR1, &signalaction, NULL) < 0)
-    	die(_("Cannot register signal handler - %s\n"), errstr);
+    	die(_("cannot register signal handler - %s\n"), errstr);
 
     if (sigemptyset(&signalset) < 0)
-    	die(_("Cannot empty signal set - %s\n"), errstr);
+    	die(_("cannot empty signal set - %s\n"), errstr);
     if (sigaddset(&signalset, SIGTERM) < 0)
-    	die(_("Cannot add signal to set - %s\n"), errstr);
+    	die(_("cannot add signal to set - %s\n"), errstr);
     if (sigaddset(&signalset, SIGINT) < 0)
-    	die(_("Cannot add signal to set - %s\n"), errstr);
+    	die(_("cannot add signal to set - %s\n"), errstr);
     if (sigaddset(&signalset, SIGUSR1) < 0)
-    	die(_("Cannot add signal to set - %s\n"), errstr);
+    	die(_("cannot add signal to set - %s\n"), errstr);
     if (sigprocmask(SIG_BLOCK, &signalset, NULL) < 0)
-    	die(_("Cannot block signals - %s\n"), errstr);
+    	die(_("cannot block signals - %s\n"), errstr);
 
     if (!init_metadata(argv+optind, argc-optind)) {
         finish_logging(false);
@@ -359,7 +394,10 @@ main(int argc, char **argv)
     finish_metadata();
 
     if (pidfilename != NULL)
-    	unlink(pidfilename);	/* ignore errors */
+    	unlink(pidfilename); /* ignore errors */
+
+    if (charset_convert != (iconv_t) -1)
+        iconv_close(charset_convert); /* ignore errors (only EINVAL) */
 
     finish_logging(true);
 
