@@ -231,7 +231,7 @@ handle_action_request(struct Upnp_Action_Request *request)
 		ixmlFreeDOMString(dump);
 	    }
 
-	    say(2, _("Action executed successfully\n"));
+	    say(2, _("Action succeeded.\n"));
 	} else {
 	    say(2, _("Action failed.\n"));
 	    say(3, _("Error code: %d\n"), event.request->ErrCode);
@@ -399,23 +399,23 @@ init_upnp(const char *listenip, uint16_t listenport)
     say(3, _("Initializing UPnP subsystem...\n"));
     res = UpnpInit(listenip, listenport);
     if (res != UPNP_E_SUCCESS)
-        die(_("Cannot initialize UPnP subsystem - %s\n"), upnp_errmsg(res));
+        die(_("cannot initialize UPnP subsystem - %s\n"), upnp_errmsg(res));
 
     say(1, _("UPnP MediaServer listening on %s:%d\n"), UpnpGetServerIpAddress(), UpnpGetServerPort());
 
     say(3, _("Enabling UPnP web server...\n"));
     res = UpnpEnableWebserver(TRUE);
     if (res != UPNP_E_SUCCESS)
-        die(_("Cannot enable UPnP web server - %s\n"), upnp_errmsg(res));
+        die(_("cannot enable UPnP web server - %s\n"), upnp_errmsg(res));
     res = UpnpSetVirtualDirCallbacks(&virtual_dir_callbacks);
     if (res != UPNP_E_SUCCESS)
-        die(_("Cannot set virtual directory callbacks - %s\n"), upnp_errmsg(res));
+        die(_("cannot set virtual directory callbacks - %s\n"), upnp_errmsg(res));
     res = UpnpAddVirtualDir("/files");
     if (res != UPNP_E_SUCCESS)
-        die(_("Cannot add virtual directory for web server - %s\n"), upnp_errmsg(res));
+        die(_("cannot add virtual directory for web server - %s\n"), upnp_errmsg(res));
     res = UpnpAddVirtualDir("/upnp");
     if (res != UPNP_E_SUCCESS)
-        die(_("Cannot add virtual directory for web server - %s\n"), upnp_errmsg(res));
+        die(_("cannot add virtual directory for web server - %s\n"), upnp_errmsg(res));
 
     say(3, _("Registering UPnP root device...\n"));
 
@@ -427,13 +427,13 @@ init_upnp(const char *listenip, uint16_t listenport)
               strlen(mediaserver_desc), 1, device_callback_event_handler, NULL, &device);
     free(mediaserver_desc);
     if (res != UPNP_E_SUCCESS)
-        die(_("Cannot register root device - %s\n"), upnp_errmsg(res));
+        die(_("cannot register root device - %s\n"), upnp_errmsg(res));
 
     say(1, _("Sending UPnP advertisement for device (expire time %d %s)...\n"),
             ssdp_expire_time, ngettext("second", "seconds", ssdp_expire_time));
     res = UpnpSendAdvertisement(device, ssdp_expire_time);
     if (res != UPNP_E_SUCCESS)
-        die(_("Cannot send device advertisement - %s\n"), upnp_errmsg(res));
+        die(_("cannot send device advertisement - %s\n"), upnp_errmsg(res));
 
     say(1, _("Listening for control point connections...\n"));
 }
@@ -446,10 +446,10 @@ finish_upnp(void)
     say(1, _("Shutting down UPnP MediaServer...\n"));
     res = UpnpUnRegisterRootDevice(device);
     if (res != UPNP_E_SUCCESS)
-        die(_("Cannot unregister root device - %s\n"), upnp_errmsg(res));
+        die(_("cannot unregister root device - %s\n"), upnp_errmsg(res));
 
     say(3, _("Deinitializing UPnP subsystem...\n"));
     res = UpnpFinish();
     if (res != UPNP_E_SUCCESS)
-        die(_("Cannot deinitialize UPnP library - %s\n"), upnp_errmsg(res));
+        die(_("cannot deinitialize UPnP library - %s\n"), upnp_errmsg(res));
 }
